/*
 * Copyright (C) 2007-2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>

#include <gmodule.h>
#include <libxfce4util/libxfce4util.h>

#include "appstarter.h"

static void save (GtkWidget *b, ASChooser *chooser);

int main (int argc, char *argv[])
{
    GtkWidget *window, *chooser;
    GtkWidget *bbox, *box, *ok;
    GtkWidget *d;
    XfceKiosk *kiosk;
    gchar *conffile = NULL;
    gchar *select = NULL;
    gboolean selected = FALSE;
    gchar *with_ss = NULL;
    gchar *command = NULL;

    GOptionEntry options[] = {
        { "conffile", 'c', 0, G_OPTION_ARG_STRING,
          &conffile, N_("Use the specified configuration file"), "FILE" },
        { "select", 's', 0, G_OPTION_ARG_STRING,
          &select, N_("Set the selected item to be PLUGIN and exit"), "PLUGIN" },
        { "command", 'r', 0, G_OPTION_ARG_STRING,
          &command, N_("Set the command to be CMD and exit"), "CMD" },
        { "selected", 'w', 0, G_OPTION_ARG_NONE,
          &selected, N_("Show the plugin currently selected"), NULL },
        { "with-ss", 's', 0, G_OPTION_ARG_STRING,
          &with_ss, N_("If BOOL is \"false\", disable the screen saver"),"BOOL" },

        { NULL },
    };

    if (!gtk_init_with_args (&argc, &argv, "", options, PACKAGE, NULL)) {
        g_print ( _(
            "%s:\nTry %s --help to see a full list of available command line options.\n"),
                PACKAGE, PACKAGE_NAME);
        return 1;
    }

    kiosk = xfce_kiosk_new ("AppStarter");
    if (!xfce_kiosk_query (kiosk, "Allowed") && (setuid (0) != 0))
    {
        d = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL,
                       GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                       _("You are not allowed to run this application."));
        gtk_dialog_run (GTK_DIALOG (d));
        goto out;
    }

    if (! conffile)
        conffile = xfce_get_homefile (".config", "appstarterrc", NULL);

    if (selected)
    {
        XfceRc *rc= as_rc_open (conffile, TRUE);
        xfce_rc_set_group (rc, "default");

        printf ("%s", xfce_rc_read_entry (rc, "selected", ""));
        xfce_rc_close (rc);
        goto out;
    }

    if (select || command || with_ss)
    {
        XfceRc *rc = as_rc_open (conffile, FALSE);
        xfce_rc_set_group (rc, "default");

        if (with_ss)
        {
            xfce_rc_write_bool_entry (rc, "noscreensaver",
                                      g_strcmp0 (with_ss, "false") == 0);
            g_free (with_ss);
        }
        if (select)
        {
            xfce_rc_write_entry (rc, "selected", select);
            g_free (select);
        }
        if (command)
        {
            xfce_rc_write_entry (rc, "command", command);
            g_free (command);
        }

        xfce_rc_close (rc);
        goto out;
    }

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), "AppStarter");
    gtk_window_set_default_icon_name ("system-run");
    gtk_container_set_border_width (GTK_CONTAINER (window), 5);

    box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
    gtk_container_add (GTK_CONTAINER (window), box);

    chooser = as_chooser_new (conffile);
    g_free (conffile);
    gtk_box_pack_start (GTK_BOX (box), chooser, TRUE, TRUE, 6);

    bbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
    gtk_button_box_set_layout (GTK_BUTTON_BOX (bbox), GTK_BUTTONBOX_END);
    ok = gtk_button_new_from_stock (GTK_STOCK_OK);
    gtk_box_pack_end (GTK_BOX (bbox), ok, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (box), bbox, FALSE, FALSE, 6);

    g_signal_connect (G_OBJECT (ok), "clicked",
                      G_CALLBACK (save), chooser);
    g_signal_connect (G_OBJECT (window), "destroy",
                      G_CALLBACK (gtk_main_quit), NULL);

    gtk_widget_show_all (window);

    gtk_main ();

out:
    xfce_kiosk_free (kiosk);
    return 0;
}

static void
save (GtkWidget *b, ASChooser *chooser)
{
    as_chooser_save (chooser);
    gtk_main_quit ();
}

